#!/usr/bin/perl --
#
#	Copyright 2003-2004 Groundwork Open Source Solution.
#	http://www.itgroundwork.com
#
#	Unless required by applicable law or agreed to in writing, software
#	distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#	WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#	License for the specific language governing permissions and limitations under
#	the License.
#

# Relevenant Nagios V2 config file definitions:
#
# In nagios.cfg:
#
# Enable Performance Data. Make sure service/host definitions don't override this.
#		process_performance_data=1
#
# Designate the host and service performance handler programs:
#		host_perfdata_command=process-host-perfdata
#		service_perfdata_command=process-service-perfdata
#
#
# In checkcommands.cfg you will need to define command names to process the service and host perf data
#
#define command{
#	command_name	process-host-perfdata
#	command_line	/usr/local/nagios/eventhandlers/nsca_perfdata_handler.pl HOST $HOSTNAME$ $LASTHOSTCHECK$ "$HOSTOUTPUT$" $HOSTSTATEID$ $HOSTPERFDATA$
#	}
#
##define command{
#	command_name	process-service-perfdata
#	command_line	/usr/local/nagios/eventhandlers/nsca_perfdata_handler.pl SERVICE $HOSTNAME$ "$SERVICEDESC$" $LASTSERVICECHECK$ "$SERVICEOUTPUT$" $SERVICESTATEID$ "$SERVICEPERFDATA$"
#	}

#use strict;
use Time::Local;
use IO::Socket;

# Set host to forward to (usually localhost)
my $forward_host = "localhost";

my $debug = 0;
my $debuglog = ">> /usr/local/groundwork/nagios/eventhandlers/process_service_perf.log";


my $msgtype		= shift || &usage(%ERRORS);
if ($debug) {
        open(FP, $debuglog) ;
}
print FP "msgtype=$msgtype\n";

if ($msgtype =~ /HOST/)
{
	my $return_code="0";

	my $host = shift || &usage(%ERRORS);
	my $timestamp = shift || &usage(%ERRORS);
	my $status = shift  || &usage(%ERRORS);
	my $return_code = shift || "0";
	my $perfdata = shift || "";

	my $send_string = "echo \"";
	$send_string .= $host . "\t";
	$send_string .= $return_code . "\t";
	$send_string .= $status . "\t";
	$send_string .= $perfdata . " \" | ";

	$send_string .= "/usr/local/groundwork/nscafe/send_nscafe.pl -H $forward_host -k HOST_ALERT\n";
	print FP "send_string=$send_string\n";

	my $retcode = `$send_string`;
}
elsif ($msgtype =~ /SERVICE/)
{
	my $return_code="0";

	my $host = shift || &usage(%ERRORS);
	my $svcdesc = shift || &usage(%ERRORS);
	my $timestamp = shift || &usage(%ERRORS);
	my $status = shift  || &usage(%ERRORS);
	my $return_code = shift || "0";
	my $perfdata = shift || "";

	my $send_string = "echo \"";
	$send_string .= $host . "\t";
	$send_string .= $svcdesc . "\t";
	$send_string .= $return_code . "\t";
	$send_string .= $status . "\t";
	$send_string .= $perfdata . " \" | ";

	$send_string .= "/usr/local/groundwork/nscafe/send_nscafe.pl -H $forward_host -k SERVICE_ALERT\n";
	print FP "send_string=$send_string\n";

	my $retcode = `$send_string`;
}
else
{
	print FP "Invalid message type $msgtype\n";
	exit;
}

sub time_text
{
		my $timestamp = shift;
		if ($timestamp <= 0) {
			return "none";
		} else {
			my ($seconds, $minutes, $hours, $day_of_month, $month, $year,$wday, $yday, $isdst) = localtime($timestamp);
			return sprintf "%02d/%02d/%02d %02d:%02d:%02d",$month+1,$day_of_month,$year+1900,$hours,$minutes,$seconds;
		}
}

sub FormatTime {
	my $intimestring=shift;
	my $outtimestring;
	if ($intimestring =~ /(\d{2})\/(\d{2})\/(\d{4}) (\d{2}:\d{2}:\d{2})/) {
		$outtimestring = "$3-$1-$2 $4";
	}
	return $outtimestring;
}

sub usage {
   print FP "Required arguments not given!\n\n";
   print FP "Performance Data handler/GroundWork Collage performance data feeder for Nagios, V2.0\n";
   print FP "Copyright (c) 2004 Groundwork Open Source Solutions, All Rights Reserved \n\n";
   print FP "Usage: \n";
   print FP "		nscafe_perfdata_handler.pl HOST <host> <timestamp> <status> <output> <perfdata>\n";
   print FP "		nscafe_perfdata_handler.pl SERVICE <host> <svc description> <timestamp> <status> <output> <perfdata>\n";
   exit $ERRORS{"UNKNOWN"};
}

__END__

