package org.groundwork.foundation.mq;

import java.io.*;
import java.net.InetAddress;
import java.net.Socket;
import java.util.*;
import java.util.regex.Pattern;
import org.groundwork.foundation.mq.*;

public class nscafeConfig
{
    public boolean isConfigured()
    {
        return isConfigured;
    }

    private void isConfigured(boolean i)
    {
        isConfigured = i;
    }

    public nscafeConfig(String args[])
    {
        Properties properties = new Properties();
        isConfigured(false);
        
        if (args.length > 0)
            NSCAFE_HOME = args[0];
        else
            NSCAFE_HOME = "/usr/local/groundwork/nscafe";
        
        NSCAFE_CFG_FN = NSCAFE_HOME + "/nscafe.cfg";
        NSCAFE_DFLT_LOCAL_STOR = NSCAFE_HOME + "/nscafe";
        
        try
        {
            FileInputStream propFileStream = new FileInputStream(new File(NSCAFE_CFG_FN));
            properties.load(propFileStream);
            propFileStream.close();
            debugLevel = Integer.parseInt(properties.getProperty("debug", "0")) == 1;
            forwardMode = Integer.parseInt(properties.getProperty("forward", "0")) == 1;
            nagiosMode = Integer.parseInt(properties.getProperty("nagios", "")) == 1;
            listenerPort = Integer.parseInt(properties.getProperty("listener_port", NSCAFE_DFLT_PORT));
            allowedHosts = properties.getProperty("allowed_hosts", "*");
            configurationParseAllowedHosts(allowedHosts);
            clientInactivityTimeout = Integer.parseInt(properties.getProperty("client_inactivity_timeout", "5"));
            if(nagiosMode)
                nagiosCommandFile = properties.getProperty("nagios_command_file", NSCAFE_DFLT_NAG_CMD);
            if(forwardMode)
            {
                forwardHost = properties.getProperty("forward_host", "localhost");
                forwardPort = Integer.parseInt(properties.getProperty("forward_port", NSCAFE_DFLT_PORT));
                forwardAllowedServices = properties.getProperty("forward_allowed_services", "*");
                forwardStateChangeOnly = Integer.parseInt(properties.getProperty("forward_state_change_only", "0")) == 1;
                forwardFixedConnection = Integer.parseInt(properties.getProperty("forward_fixed_connection", "1")) == 1;
                forwardRemovePerfdata = Integer.parseInt(properties.getProperty("forward_remove_perfdata", "0")) == 1;
                configurationParseForwardAllowedServices(forwardAllowedServices);
                queueDirectory = properties.getProperty("forward_queue_directory", NSCAFE_DFLT_LOCAL_STOR);
                queueFilename = queueDirectory + "/nscafe.dat";
                queuePurgeOnEntry = Integer.parseInt(properties.getProperty("forward_purge_queue_on_entry", "")) == 1;
                
            }
            threaded = false;
            isConfigured(true);
            dump();
        }
        catch(FileNotFoundException ex)
        {
            System.out.println("nscafe: Can't open configuration file " + NSCAFE_CFG_FN);
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }
    }

    public void configurationParseAllowedHosts(String ah)
    {
        String token;
        for(StringTokenizer st = new StringTokenizer(ah, ","); st.hasMoreTokens(); allowedHostsList.add(token))
        {
            token = st.nextToken();
            token.trim();
        }
    }
    
     public void configurationParseForwardAllowedServices(String as)
    {
        String token;
        for(StringTokenizer st = new StringTokenizer(as, ","); st.hasMoreTokens(); forwardAllowedServicesList.add(token))
        {
            token = st.nextToken();
            token.trim();
        }
    }

    public void dump()
    {
        System.out.println("NSCAFE CONFIGURATION DUMP");
        System.out.println("FILE: " + NSCAFE_CFG_FN);
        System.out.println("  GENERAL");
        System.out.println("    debugLevel = " + debugLevel);
        System.out.println("    nagiosMode = " + nagiosMode);
        System.out.println("    forwardMode = " + forwardMode);
        System.out.println("  LISTENER");
        System.out.println("    listenerPort = " + listenerPort);
        System.out.println("    allowedHosts = " + allowedHosts);
        System.out.println("    clientInactivityTimeout = " + clientInactivityTimeout);
        System.out.println("  NAGIOS");
        System.out.println("    nagiosCommandFile = " + nagiosCommandFile);
        System.out.println("  FORWARDING");
        System.out.println("    forwardHost = " + forwardHost);
        System.out.println("    forwardPort = " + forwardPort);
        System.out.println("    forwardAllowedServices = " + forwardAllowedServices);
        System.out.println("    forwardStateChangeOnly = " + forwardStateChangeOnly);
        System.out.println("    forwardRemovePerfdata = " + forwardRemovePerfdata);
        System.out.println("    forwardFixedConnection = " + forwardFixedConnection);
        System.out.println("    queueDirectory = " + queueDirectory);
        System.out.println("    queuePurgeOnEntry = " + queuePurgeOnEntry);
        System.out.println();
    }

    public static boolean isHostAllowed(Socket clientSocket)
    {
        Iterator it = allowedHostsList.iterator();
        String clientAddress = clientSocket.getInetAddress().getHostAddress();
        while(it.hasNext()) 
        {
            String allowedHost = it.next().toString();
            if (allowedHost.equals("*"))
                return(true);
            if(Pattern.matches(allowedHost, clientAddress))
                return(true);
        }
        System.out.println("nscafeConfig.isHostAllowed: Denying service to " + clientAddress);
        return(false);
    }
    
     public static boolean isForwardServiceAllowed(String serviceDescription)
    {
        Iterator it = forwardAllowedServicesList.iterator();
        while(it.hasNext()) 
        {
            String forwardAllowedService = it.next().toString();
            if (forwardAllowedService.equals("*"))
                return(true);
            if(Pattern.matches(forwardAllowedService, serviceDescription))
                return true;
        }
        return false;
    }

    private static String NSCAFE_HOME;
    private static String NSCAFE_CFG_FN;
    private static String NSCAFE_DFLT_PORT = "5668";
    private static String NSCAFE_DFLT_NAG_CMD = "/usr/local/groundwork/nagios/var/spool/nagios.cmd";
    private static String NSCAFE_DFLT_LOCAL_STOR;
    private boolean isConfigured;
    public boolean debugLevel;
    public boolean forwardMode;
    public boolean nagiosMode;
    public int listenerPort;
    public String allowedHosts;
    public static ArrayList allowedHostsList = new ArrayList();
    public long clientInactivityTimeout;
    public String nagiosCommandFile;
    public String forwardHost;
    public int forwardPort;
    public String forwardAllowedServices;
    public static ArrayList forwardAllowedServicesList = new ArrayList();
    public boolean forwardStateChangeOnly;
    public boolean forwardFixedConnection;
    public boolean forwardRemovePerfdata;
    public String queueDirectory;
    public String queueFilename;
    public boolean queuePurgeOnEntry;
    public boolean threaded;
}
