package org.groundwork.foundation.mq;

import java.io.*;
import java.net.Socket;
import java.net.InetSocketAddress;
import java.net.SocketTimeoutException;
import org.groundwork.foundation.mq.nscafeConfig;
import org.groundwork.foundation.mq.*;

public class nscafeForward
{
    private nscafeServer    server;
    public Socket           forwardServerSocket;
    public OutputStream     forwardServerOutputStream;
    
    nscafeForward(nscafeServer ni)
    {
        server                      = ni;
        forwardServerSocket         = null;
        forwardServerOutputStream   = null;
    }
    
    public boolean isForwardConnectionOpen()
    {
         return(forwardServerSocket != null);
    }

    public boolean writeToForward(String xml)
    {
        boolean returnCode = false;
        
        if (!isForwardConnectionOpen())
            openConnectionToForward();
        
        if(isForwardConnectionOpen() && forwardServerSocket.isConnected() && !forwardServerSocket.isClosed())
        {
            String xmlStream = xml + "\n";
        
            try
            {
                forwardServerOutputStream.write(xmlStream.getBytes());
                returnCode = true;
            }
            catch(IOException e)
            {
                closeConnectionWithForward();
                System.out.println("nscafeForward.writeToForward(): Error while writing to forward server, considering it to be down, will re-try on next write attempt.");
                if(server.config.debugLevel)
                    e.printStackTrace();
            }
        }
        else
            System.out.println("nscafeForward.writeToForward(): Not connected to forward server, unable to send packet.");
        
        if (!server.config.forwardFixedConnection)
            closeConnectionWithForward();
        
        return(returnCode);
    }
    
    public boolean openConnectionToForward()
    {
       try
        {
            System.out.println("forwardHost=" + server.config.forwardHost + ",port=" + server.config.forwardPort);
            
            forwardServerSocket = new Socket(server.config.forwardHost, server.config.forwardPort);
            
            //forwardServerSocket.setSoTimeout((int)(listenerInstance.config.clientInactivityTimeout * 100L));
            //forwardServerSocket.setSoLinger(true, 0);
            
            forwardServerOutputStream = forwardServerSocket.getOutputStream();
            
            return(true);
        }
        catch(IOException e)
        {
            System.out.println("nscafeForward.openConnectionToForward(): Failed to connect to forward host");
            if(server.config.debugLevel)
                e.printStackTrace();
            
            closeConnectionWithForward();
            return(false);
        }
    }

    public void closeConnectionWithForward()
    {
        if(forwardServerSocket != null)
            try
            {
                forwardServerSocket.close();
                forwardServerSocket = null;
                forwardServerOutputStream.close();
                forwardServerOutputStream = null;
                
            }
            catch(IOException e)
            {
                forwardServerOutputStream = null;
                forwardServerSocket = null;
                if(server.config.debugLevel)
                {
                    System.out.println("nscafeForward.forwardCloseServerConnection(): couldn't close connection to forward host " + server.config.forwardHost + ":" + server.config.forwardPort);
                    e.printStackTrace();
                }
            }
    }
}
