package org.groundwork.foundation.mq;

import java.io.*;
import java.util.Hashtable;
import java.util.concurrent.locks.ReentrantLock;

import org.groundwork.foundation.mq.nscafeServer;
import org.groundwork.foundation.mq.nscafeXML;
import org.groundwork.foundation.mq.*;

public class nscafeNagios
{
    private FileOutputStream    nagiosCommandFileStream;
    private boolean             isGood;
    private final ReentrantLock lock = new ReentrantLock();
    private nscafeServer        nscafeInstance;    
    
    public boolean isGood() { return(isGood); };
    
    public nscafeNagios(nscafeServer ni)
    {
        nscafeInstance = ni;
        isGood = openCommandFile();
    }
    
    public boolean openCommandFile()
    {
        try
        {
            nagiosCommandFileStream = new FileOutputStream(nscafeInstance.config.nagiosCommandFile, false);
        }
        catch(FileNotFoundException ex)
        {
            System.out.println("nscafeNagios.openCommandFile: Unable to open Nagios Command file at " + nscafeInstance.config.nagiosCommandFile);
            if(nscafeInstance.config.debugLevel)
                ex.printStackTrace();
        }
        return true;
    }

    public void closeCommandFile()
    {
        try
        {
            nagiosCommandFileStream.close();
            nagiosCommandFileStream = null;
        }
        catch(IOException e)
        {
            System.out.println("nscafeNagios.closeCommandFile: Unable to close nagios command file " + nscafeInstance.config.nagiosCommandFile);
            if(nscafeInstance.config.debugLevel)
                e.printStackTrace();
        }
        finally
        {
            nagiosCommandFileStream = null;
        }
    }

    public void write(Hashtable xmlAttributes)
    {
        try
        {
            //lock.lock();
            if (nagiosCommandFileStream == null)
                if (!openCommandFile())
                    return;

            try
            {
                String commandString;
                if((commandString = createEntry(xmlAttributes)) != null)
                    nagiosCommandFileStream.write(commandString.getBytes());
            }
            catch(IOException e)
            {
                System.out.println("nscafeNagios.write: Can't write to nagios command file " + nscafeInstance.config.nagiosCommandFile);
                closeCommandFile();
                if(nscafeInstance.config.debugLevel)
                    e.printStackTrace();
            }
        }
        finally
        {
            //lock.unlock();
        }
    }

    public String createEntry(Hashtable attribs)
    {
        String errorType        = nscafeXML.getAttribute(attribs, "ErrorType");
        String lastInsertDate   = nscafeXML.getAttribute(attribs, "LastInsertDate");
        StringBuffer cmdInsert  = new StringBuffer();
            
        if (errorType.compareToIgnoreCase("EXECUTE_COMMAND") == 0)
        {
            String command  = nscafeXML.getAttribute(attribs, "Command");
            String args     = nscafeXML.getAttribute(attribs, "Args");
            
            dumpCommandEntry(attribs);
            cmdInsert.append("[").append(lastInsertDate).append("] ").append(command).append(";").append(args).append("\n");
        }
        else
        {
            String severity = nscafeXML.getAttribute(attribs, "Severity");
            String host = nscafeXML.getAttribute(attribs, "Host");
            String description = nscafeXML.getAttribute(attribs, "ServiceDescription");
            String fullTextMessage = nscafeXML.getAttribute(attribs, "TextMessage"), textMessage;
            String returnCode;
            
            if(severity == null || severity == "" || severity == "-")
                returnCode = "3";
            else
                returnCode = severity;
            
            if (nscafeInstance.config.forwardRemovePerfdata)
            {
                int pipeChar = fullTextMessage.indexOf('|');
                
                if (pipeChar > 1)
                    textMessage = fullTextMessage.substring(0,pipeChar-1);
                else
                    textMessage = "";
            }
            else
                textMessage = fullTextMessage;

            attribs.put("TextMessage", textMessage);
            
            if(errorType.compareToIgnoreCase("SERVICE_ALERT") == 0)
            {
                dumpAlertEntry(attribs, "PROCESS_SERVICE_CHECK_RESULT", returnCode);
                cmdInsert.append("[").append(lastInsertDate).append("] ").append("PROCESS_SERVICE_CHECK_RESULT").append(";").append(host).append(";").append(description).append(";").append(returnCode).append(";").append(textMessage).append("\n");
            }
            else
            {
                dumpAlertEntry(attribs, "PROCESS_HOST_CHECK_RESULT", returnCode);
                cmdInsert.append("[").append(lastInsertDate).append("] ").append("PROCESS_HOST_CHECK_RESULT").append(";").append(host).append(";").append(returnCode).append(";").append(textMessage).append("\n");
            }
        }
        return(cmdInsert.toString());
    }

    void dumpAlertEntry(Hashtable attribs, String commandType, String returnCode)
    {
        if(nscafeInstance.config.debugLevel)
        {
            System.out.println("{");
            System.out.println("    CommandType = " + commandType);
            System.out.println("    ServiceDescription = " + nscafeXML.getAttribute(attribs, "ServiceDescription"));
            System.out.println("    Host = " + nscafeXML.getAttribute(attribs, "Host"));
            System.out.println("    TextMessage = " + nscafeXML.getAttribute(attribs, "TextMessage"));
            System.out.println("    LastInsertDate = " + nscafeXML.getAttribute(attribs, "LastInsertDate"));
            System.out.println("    returnCode = " + returnCode);
            System.out.println("}");
        }
    }
    
    void dumpCommandEntry(Hashtable attribs)
    {
        if(nscafeInstance.config.debugLevel)
        {
            System.out.println("{");
            System.out.println("    Command = " + nscafeXML.getAttribute(attribs, "Command"));
            System.out.println("    Args = " + nscafeXML.getAttribute(attribs, "Args"));
            System.out.println("    LastInsertDate = " + nscafeXML.getAttribute(attribs, "LastInsertDate"));
            System.out.println("}");
        }
    }
}
