package org.groundwork.foundation.mq;

import java.io.*;
import org.groundwork.foundation.mq.*;
import org.groundwork.foundation.mq.nscafeServer;

// Referenced classes of package org.groundwork.foundation.mq:
//            nscafe, nscafeConfig, nscafeNonPersistentForward

public class nscafeQueue
{
    private File            queueFile;
    private BufferedWriter  queueOutputStream;
    private boolean         hasEntries;
    private nscafeServer    listenerInstance;
    private boolean         isOpen = false;
    
    public boolean isOpen() { return(isOpen); };
    
    public nscafeQueue(nscafeServer ni)
    {
        listenerInstance = ni;
        hasEntries = false;
        isOpen = false;
    }

    public boolean hasEntries()
    {
        return hasEntries;
    }

    public boolean write(String xmlStream)
    {
        if (!isOpen)
            if (!openQueue())
                return(false);
        
        if(listenerInstance.config.debugLevel)
            System.out.println("nscafeQueue: writing xml packet to queue.");
        try
        {
            queueOutputStream.write(xmlStream + "\n");
            queueOutputStream.flush();
            
            return(hasEntries = true);
        }
        catch(IOException e)
        {
            System.out.println("nscafeQueue: Error writing to queue file " + listenerInstance.config.queueFilename);
            if(listenerInstance.config.debugLevel)
                e.printStackTrace();
            return(false);
        }
    }

    public void dumpQueueToForward(nscafeForward forward)
    {
        System.out.println("nscafeQueue: Dumping queue to forward host.");
        if(hasEntries() && openQueue())
        {
            BufferedReader queueInputStream = null;
            boolean eof = false;
            try
            {
                queueInputStream = new BufferedReader(new FileReader(queueFile));
                while(!eof) 
                {
                    String xmlStream = queueInputStream.readLine();
                    if(xmlStream == null)
                    {
                        eof = true;
                    } else
                    {
                        xmlStream.trim();
                        forward.writeToForward(xmlStream);
                    }
                }
                queueInputStream.close();
                purgeQueue();
            }
            catch(FileNotFoundException e)
            {
                System.out.println("nscafeQueue: Can't find queue file to dump to forward host.");
                if(listenerInstance.config.debugLevel)
                    e.printStackTrace();
            }
            catch(IOException e)
            {
                if(queueInputStream != null)
                    try
                    {
                        queueInputStream.close();
                    }
                    catch(IOException ioexception) { }
            }
        }
    }

    public void purgeQueue()
    {
        closeQueue();
        queueFile.delete();
        openQueue();
        hasEntries = false;
    }

    public boolean openQueue()
    {
        if (isOpen)
            return(true);
        
        queueFile = new File(listenerInstance.config.queueFilename);
        queueOutputStream = null;
        hasEntries = false;
        try
        {
            queueOutputStream = new BufferedWriter(new FileWriter(queueFile, !listenerInstance.config.queuePurgeOnEntry));
            if(!listenerInstance.config.queuePurgeOnEntry && queueFile.length() != 0L)
                hasEntries = true;
            return (isOpen = true);
        }
        catch(IOException e)
        {
            queueOutputStream = null;
            System.out.println("nscafe: Can't open queue file " + listenerInstance.config.queueFilename);
            if(listenerInstance.config.debugLevel)
                e.printStackTrace();
            return (isOpen = false);
        }
    }

    public void closeQueue()
    {
        if (isOpen)
        {
            try
            {
                isOpen = false;
                queueOutputStream = null;
                hasEntries = false;

                if(queueOutputStream != null)
                    queueOutputStream.close();
            }
            catch(IOException e)
            {
                System.out.println("nscafe: Error closing queue file " + listenerInstance.config.queueFilename);
                if(listenerInstance.config.debugLevel)
                    e.printStackTrace();
            }
        }
    }
}
