package org.groundwork.foundation.mq;

import groundwork.foundation.mq.nscafeSignalHandler;
import java.io.*;
import java.net.*;

import org.groundwork.foundation.mq.nscafeConfig;
import org.groundwork.foundation.mq.nscafeNagios;
import org.groundwork.foundation.mq.nscafeForward;
import java.util.Hashtable;
import java.util.Observer;
import java.util.Observable;

//import org.groundwork.foundation.mq.nscafeServer;

// nscafeListener
//
// There is a single instance of nscafeListener.
//

public class nscafeServer implements Observer
{
    // Instance variables.
    
    public nscafeConfig         config = null;
    public nscafeNagios         nagios = null;
    public nscafeForward        forward = null;    
    public nscafeQueue          queue = null;
    public Hashtable            serviceCollection = null;
    private boolean             isGood = false;
    private boolean             exiting = false;
    
    // Various constructors and selectors.
    
    public boolean isGood() { return(isGood); };
    
    // Main code.
    
    public nscafeServer(String args[])
    {
        serviceCollection   = new Hashtable();
        config              = new nscafeConfig(args);
        isGood              = true;
        exiting             = false;
        
        if(config.isConfigured())
        {
            if(config.nagiosMode)
            {
                nagios = new nscafeNagios(this);
                isGood = nagios.isGood();
            }
        
            if (config.forwardMode)
            {
                forward = new nscafeForward(this);
                queue = new nscafeQueue(this);
            }
        }
        else
            System.out.println("nscafeListener.run(): cannot get configuration information.");
    }
    
    private void cleanup()
    {
        if(config.nagiosMode && nagios != null)
            nagios.closeCommandFile();
        
        if (config.forwardMode)
        {
            if (forward != null)
                forward.closeConnectionWithForward();
            
            if (queue != null)
                queue.closeQueue();
        }            
    }
    
    public void run()
    {
        if (isGood())
            listener();
    }

    private void listener()
    {      
        ServerSocket                serverSocket = null;
        Socket                      clientSocket = null;
        final nscafeSignalHandler   sigh = new nscafeSignalHandler();
        
        sigh.addObserver(this);
        //sigh.handleSignal("KILL");
        sigh.handleSignal("HUP");
        
        System.out.println("nscafeListener.listener(): opening server socket localhost:" + config.listenerPort);
        try
        {
            serverSocket = new ServerSocket(config.listenerPort, 8192);
            try 
            {
                System.out.println("nscafeListener.listener(): listening for incoming connections...");
                
                while (!exiting)
                {
                    clientSocket = serverSocket.accept();
                    if(nscafeConfig.isHostAllowed(clientSocket))
                    {
                        if(config.debugLevel)
                            System.out.println("nscafeListener.listener(): accepted connection request from allowed host " + clientSocket.getInetAddress().getHostAddress());
                        
                        //clientSocket.setSoTimeout((int)(config.clientInactivityTimeout * 100L));
                        //clientSocket.setSoLinger(true, 0);
                        
                        nscafeThread    server = new nscafeThread(clientSocket, this);
                        //Thread          t = new Thread(server);
                        //t.start();
                        server.run();
                    }
                    else
                        clientSocket.close();
                }
                cleanup();
            }
            catch(IOException e)
            {
                if(config.debugLevel)
                {
                    System.out.println("nscafeListener.listener(): accept failed: " + config.listenerPort);
                    e.printStackTrace();
                }
            }
        }
        catch(IOException e)
        {
            System.out.println("nscafeListener.listener(): Could not listen on port: " + config.listenerPort);
            if(config.debugLevel)
                e.printStackTrace();
        }
    }
    
    public void update(final Observable o, final Object arg)
    {
        System.out.println("nscafeServer.update(): Received signal: " + arg + ", cleaning up and exiting.");
        exiting = true;
    }
    
    public int getServiceState(String hostName, String serviceName)
    {
        String uniqueName = hostName + serviceName;
        Integer state;
        
        state = (Integer)serviceCollection.get(uniqueName);
        if (state != null)
            return(state.intValue());
        else
            return(-1);
    }
    
    public void putServiceState(String hostName, String serviceName, int serviceState)
    {
        String uniqueName = hostName + serviceName;
        Integer state = new Integer(serviceState);
        
        serviceCollection.put(uniqueName, state);
    }
}
