package org.groundwork.foundation.mq;

/*
 * nscafeServer.java
 *
 * Created on September 14, 2006, 3:08 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import java.io.*;
import java.net.*;
import java.util.concurrent.locks.ReentrantLock;

import org.groundwork.foundation.mq.nscafeServer;
import org.groundwork.foundation.mq.nscafeConfig;
import org.groundwork.foundation.mq.nscafeNagios;
import org.groundwork.foundation.mq.nscafeForward;
import org.groundwork.foundation.mq.nscafeXML;

class nscafeThread /*implements Runnable*/
{
    private Socket                  clientSocket;
    private nscafeServer            server;
    //private final ReentrantLock   lock = new ReentrantLock();
    
   public nscafeThread(Socket client, nscafeServer ni)
   {
       clientSocket = client;
       server = ni;
   }
   
   public void run()
   {
        InputStreamReader inStream  = null;
        BufferedReader in           = null;
        
        System.out.println("nscafeThread.run(): Entering new thread with " + clientSocket.getInetAddress().getHostAddress());
        
        try
        {
            inStream = new InputStreamReader(clientSocket.getInputStream());
            in = new BufferedReader(inStream);
            
            while(clientSocket.isConnected()) 
            {
                String  xmlStream = in.readLine();
                
                if(xmlStream == null)
                    return;
               
                xmlStream.trim();
              
                System.out.println("nscafeThread.run(): Received xml packet from client.");

                if(xmlStream.length() > 0)
                {
                    if (!xmlStream.equals("<CLOSE>"))
                    {
                        java.util.Hashtable xmlAttributes = nscafeXML.getAttributes(server, xmlStream);
                        if(xmlAttributes != null)
                        {
                            if(server.config.nagiosMode)
                                server.nagios.write(xmlAttributes);
                           
                            if(server.config.forwardMode)
                            {
                                String  serviceName = nscafeXML.getAttribute(xmlAttributes, "ServiceDescription");
                                if (server.config.isForwardServiceAllowed(serviceName))
                                {
                                    if (server.config.forwardStateChangeOnly == false)
                                        writeToForward(xmlStream);
                                    else
                                    {
                                        String  hostName    = nscafeXML.getAttribute(xmlAttributes, "Host");
                                        String  severity;
                                        int     currentServiceState, previousServiceState;
                                        
                                        severity = nscafeXML.getAttribute(xmlAttributes, "Severity");
                                        
                                        if(severity == null || severity == "" || severity == "-")
                                            currentServiceState = 3;
                                        else
                                        {
                                            System.out.println("SEVERITY=" + severity);
                                            
                                            currentServiceState = Integer.parseInt(severity) ;
                                        }
                                        
                                        previousServiceState = server.getServiceState(hostName, serviceName);
                                        System.out.println("STATE OF " + hostName + ":" + serviceName + " == " + previousServiceState + ", currentServiceState == " + currentServiceState);
                                        
                                        if (previousServiceState == -1 || (previousServiceState != currentServiceState))
                                        {
                                            System.out.println("FORWARDING " + hostName + ":" + serviceName);
                                            server.putServiceState(hostName, serviceName, currentServiceState);
                                            writeToForward(xmlStream);
                                        }
                                        else
                                            System.out.println("NOT FORWARDING " + hostName + ":" + serviceName);
                                    }
                                }
                            }
                        }
                        else
                            if(server.config.debugLevel)
                                System.out.println("nscafeThread.run(): XML attribute map is null, exiting message");
                    }
                }
            }
        }
        catch(IOException e)
        {
            // Do nothing special here,
            // just fall through to the finally block.            
        }
        finally
        {
            closeClient();
            
            System.out.println("nscafeThread.run(): Exiting thread with " + clientSocket.getInetAddress().getHostAddress());
            System.out.println("--------");        
        }
    }                   
   
    private void closeClient()
    {
        try
        {
            clientSocket.close();
        }
        catch(IOException e)
        {
            System.out.println("nscafeThread.closeClient(): error closing connection with client: " + clientSocket.getInetAddress().getHostAddress());
            e.printStackTrace(); 
        }        
    }
   
    private void writeToForward(String xmlStream)
    {
       try
       {
            //lock.lock();
            if(server.forward.writeToForward(xmlStream))
            {
                if(server.queue.hasEntries())
                    server.queue.dumpQueueToForward(server.forward);
            }
            else
            {
                server.queue.write(xmlStream);
            }
        }
        finally
        {
            //lock.unlock();
        }
    }
}
